import * as plugins from './tink.plugins';

import { TinkAccount } from './tink.classes.tinkaccount';

export class TinkUser {
  // STATIC
  public static async createNewTinkUser(tinkaccountArg: TinkAccount, externalUserIdArg: string) {
    const accessToken = await tinkaccountArg.getClientAccessTokenForScope('user:create');
    const response = await tinkaccountArg.request({
      urlArg: '/api/v1/user/create',
      accessToken,
      methodArg: 'POST',
      payloadArg: {
        external_user_id: externalUserIdArg,
        market: 'DE',
        locale: 'en_US',
      },
    });

    const newTinkUser = new TinkUser(tinkaccountArg, response.user_id, response.external_user_id);
    return newTinkUser;
  }

  // INSTANCE
  public tinkAccountRef: TinkAccount;
  public tinkUserId: string;
  public externalUserIdArg: string;

  public authorizationToken: string;

  constructor(tinkAccountrefArg: TinkAccount, tinkUserIdArg: string, externalUserIdArg: string) {
    this.tinkAccountRef = tinkAccountrefArg;
    this.tinkUserId = tinkUserIdArg;
    this.externalUserIdArg = externalUserIdArg;
  }

  /**
   * deletes the user
   */
  public async delete() {
    const authorizationCode = await this.tinkAccountRef.getUserAuthorizationCode(
      this.externalUserIdArg,
      this.tinkAccountRef.clientId,
      'user:delete'
    );
    const accessToken = await this.tinkAccountRef.getUserAccessToken(authorizationCode);
    const response = await this.tinkAccountRef.request({
      methodArg: 'POST',
      accessToken,
      payloadArg: {},
      urlArg: '/api/v1/user/delete',
    });
    console.log(`successfully deleted user with externalId ${this.externalUserIdArg}`);
  }

  public async getTinkLink(): Promise<string> {
    const authorizationCode = await this.tinkAccountRef.getUserAuthorizationCode(
      this.externalUserIdArg,
      'df05e4b379934cd09963197cc855bfe9', // this is a hardcoded app id for tink link, as recommended by tink.com
      'authorization:read,authorization:grant,credentials:refresh,credentials:read,credentials:write,providers:read,user:read'
    );
    const tinkLinkUrl = `https://link.tink.com/1.0/business-transactions/connect-accounts?client_id=${'teststate'}&redirect_uri=https://console.tink.com/callback&authorization_code=${authorizationCode}&market=DE`;
    return tinkLinkUrl;
  }
}
